from django.db import models
from django.contrib.auth.models import User
import datetime
try:
    import json
except:
    import simplejson as json
from django.template.loader import get_template
from django.template import Context

class Item(models.Model):
    user = models.ForeignKey(User)
    title = models.CharField(max_length=255)
    date = models.DateField(null=True, blank=True)
    date_created = models.DateTimeField(default=datetime.datetime.now)
    json = models.TextField(blank=True)
    typ = models.ForeignKey('ItemType')    
    is_private = models.BooleanField(default=False)

    def __unicode__(self):
        return self.title

    def save_model(self, request, obj, form, change):
        obj.user = request.user
        obj.save()

    def get_fields(self):
        fields = self.typ.fields.values()
        d = json.loads(self.json)
        for f in fields:
            short_name = f['short_name']
            if d.has_key(short_name):
                f['value'] = d[short_name]
            else:
                f['value'] = ''
        return fields

    def render(self):
        template_name = self.typ.template_name
        t = get_template("types/%s" % template_name)
        context = Context(json.loads(self.json))
        return t.render(context)


class ItemType(models.Model):
    name = models.CharField(max_length=100, unique=True)
    description = models.TextField(blank=True)
    template_name = models.CharField(max_length=100)
    fields = models.ManyToManyField("ItemField")

    def __unicode__(self):
        return self.name


FIELD_TYPES = (
    ('short_text', 'Short Text Field'),
    ('large_text', 'Large Text Field'),
)

class ItemField(models.Model):
    short_name = models.CharField(max_length=255)
    description = models.CharField(max_length=255, blank=True)
    typ = models.CharField(choices=FIELD_TYPES, max_length=100)
    default_value = models.TextField(blank=True)

    def __unicode__(self):
        return "%s: %s" % (self.short_name, self.description,)


class AutoValue(models.Model):
    text = models.TextField()
    field = models.ForeignKey(ItemField)

    def __unicode__(self):
        return self.text


# Create your models here.
